//
// Copyright 2002 by Craig Stuart Sapp, All Rights Reserved.
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun Feb 10 19:42:45 PST 2002
// Last Modified: Sun Feb 10 19:42:48 PST 2002
// Filename:      ...sig/src/sigInfo/ParameterItem.cpp
// Web Address:   http://sig.sapp.org/src/sigInfo/ParameterItem.cpp
// Syntax:        C++ 
//
// Description:   Base class for SCORE musical objects.
//

#include "ParameterItem.h"

#include <string.h>

#ifndef OLDCPP
   using namespace std;
#endif



///////////////////////////////
//
// ParameterItem::ParameterItem --
//

ParameterItem::ParameterItem(void) { 
   fixedParameters.setSize(0);
   keyParameters.setSize(0);
}


ParameterItem::ParameterItem(ParameterItem& item) { 
   fixedParameters = item.fixedParameters;
   clearKeyParams();
   keyParameters.setSize(item.keyParameters.getSize());
   keyParameters.setSize(0);
   int i;
   for (i=0; i<item.keyParameters.getSize(); i++) {
      appendKeyParameter(item.keyParameters[i]);
   }
}



///////////////////////////////
//
// ParameterItem::clear --
//

void ParameterItem::clear(void) {
   fixedParameters.setSize(0);
   clearKeyParams();
}



///////////////////////////////
//
// ParameterItem::getValue -- starting offset at 1.
//

float ParameterItem::getValue(int index) { 
   float output = 0.0;
   if (index < fixedParameters.getSize()) {
      output = fixedParameters[index];
   }
   return output;
}



///////////////////////////////
//
// ParameterItem::getPValue -- starting offset at 1.
//

float ParameterItem::getPValue(int index) { 
   float output = 0.0;
   if (index-1 < fixedParameters.getSize()) {
      output = fixedParameters[index-1];
   }
   return output;
}



///////////////////////////////
//
// ParameterItem::setValue -- starting offset at 0.
//

void  ParameterItem::setValue(int index, float value) { 
   if (index < fixedParameters.getSize()) {
      fixedParameters[index] = value;
   } else {
      int oldsize = fixedParameters.getSize();
      fixedParameters.setSize(index+1);
      for (int i=oldsize; i<index+1; i++) {
         fixedParameters[i] = 0.0;
      }
      fixedParameters[index] = value;
   }
}



///////////////////////////////
//
// ParameterItem::setPValue -- starting offset at 1.
//

void  ParameterItem::setPValue(int index, float value) { 
   if (index-1 < fixedParameters.getSize()) {
      fixedParameters[index-1] = value;
   } else {
      int oldsize = fixedParameters.getSize();
      fixedParameters.setSize(index);
      for (int i=oldsize; i<index; i++) {
         fixedParameters[i] = 0.0;
      }
      fixedParameters[index-1] = value;
   }
}



///////////////////////////////
//
// ParameterItem::operator= --
//

ParameterItem& ParameterItem::operator=(ParameterItem& anItem) {
   if (&anItem == this) {
      return *this;
   }
   fixedParameters = anItem.fixedParameters;
   clearKeyParams();
   keyParameters.setSize(anItem.keyParameters.getSize());
   keyParameters.setSize(0);
   int i;
   for (i=0; i<anItem.keyParameters.getSize(); i++) {
      appendKeyParameter(anItem.keyParameters[i]);
   }

   return *this;
}



//////////////////////////////
//
// ParameterItem::print --
//

void ParameterItem::print(void) {
   int i;
   for (i=0; i<fixedParameters.getSize(); i++) {
      cout << fixedParameters[i] << " ";
   }
   cout << endl;
   for (i=0; i<keyParameters.getSize(); i++) {
      cout << keyParameters[i] << endl;
   }
}


//////////////////////////////
//
// ParameterItem::setAllocSize -- set the allocation size of the 
//   class if the current allocation size is smaller.
//

void ParameterItem::setAllocSize(int aSize) {
   if (aSize > fixedParameters.getAllocSize()) {
      int oldsize = fixedParameters.getSize();
      fixedParameters.setSize(aSize);
      fixedParameters.setSize(oldsize);
   }
}



//////////////////////////////////////////////////////////////////////////
//
// Private Functions
// 


//////////////////////////////
//
// ParameterItem::clearKeyParams --
//

void ParameterItem::clearKeyParams(void) {
   int i;
   for (i=0; i<keyParameters.getSize(); i++) {
      if (keyParameters[i] != NULL) {
         delete [] keyParameters[i];
         keyParameters[i] = NULL;
      }
   }
   keyParameters.setSize(0);
}


////////////////////////////////
//
// ParameterItem::appendKeyParameter --
//

void ParameterItem::appendKeyParameter(const char* string) {
    char *temp;
    int length = strlen(string);
    temp = new char[length+1];
    strcpy(temp, string);
    keyParameters.append(temp);
}




